@@ -1,5 +1,9 @@
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from collections.abc import Mapping
 from itertools import chain
 from typing import TYPE_CHECKING, Any, Callable, Literal, cast
 
@@ -12,7 +16,7 @@ from html_to_markdown.constants import (
     html_heading_re,
     whitespace_re,
 )
-from html_to_markdown.converters import ConvertersMap, create_converters_map
+from html_to_markdown.converters import Converter, ConvertersMap, SupportedElements, create_converters_map
 from html_to_markdown.utils import escape
 
 if TYPE_CHECKING:
@@ -85,9 +89,12 @@ def _process_tag(
     escape_misc: bool,
     escape_underscores: bool,
     strip: set[str] | None,
+    context_before: str = "",
 ) -> str:
     should_convert_tag = _should_convert_tag(tag_name=tag.name, strip=strip, convert=convert)
-    tag_name: SupportedTag | None = cast(SupportedTag, tag.name.lower()) if tag.name.lower() in converters_map else None
+    tag_name: SupportedTag | None = (
+        cast("SupportedTag", tag.name.lower()) if tag.name.lower() in converters_map else None
+    )
     text = ""
 
     is_heading = html_heading_re.match(tag.name) is not None
@@ -123,12 +130,21 @@ def _process_tag(
                 escape_misc=escape_misc,
                 escape_underscores=escape_underscores,
                 strip=strip,
+                context_before=(context_before + text)[-2:],
             )
 
     if tag_name and should_convert_tag:
-        return converters_map[tag_name](  # type: ignore[call-arg]
+        rendered = converters_map[tag_name](  # type: ignore[call-arg]
             tag=tag, text=text, convert_as_inline=convert_as_inline
         )
+        # For headings, ensure two newlines before if not already present
+        # Edge case where the document starts with a \n and then a heading
+        if is_heading and context_before not in {"", "\n"}:
+            n_eol_to_add = 2 - (len(context_before) - len(context_before.rstrip("\n")))
+            if n_eol_to_add > 0:
+                prefix = "\n" * n_eol_to_add
+                return f"{prefix}{rendered}"
+        return rendered
 
     return text
 
@@ -142,11 +158,9 @@ def _process_text(
 ) -> str:
     text = str(el) or ""
 
-    # normalize whitespace if we're not inside a preformatted element
     if not el.find_parent("pre"):
         text = whitespace_re.sub(" ", text)
 
-    # escape special characters if we're not inside a preformatted or code element
     if not el.find_parent(["pre", "code", "kbd", "samp"]):
         text = escape(
             text=text,
@@ -155,9 +169,6 @@ def _process_text(
             escape_underscores=escape_underscores,
         )
 
-    # remove trailing whitespaces if any of the following condition is true:
-    # - current text node is the last node in li
-    # - current text node is followed by an embedded list
     if (
         el.parent
         and el.parent.name == "li"
@@ -192,6 +203,8 @@ def convert_to_markdown(
     code_language: str = "",
     code_language_callback: Callable[[Any], str] | None = None,
     convert: str | Iterable[str] | None = None,
+    convert_as_inline: bool = False,
+    custom_converters: Mapping[SupportedElements, Converter] | None = None,
     default_title: bool = False,
     escape_asterisks: bool = True,
     escape_misc: bool = True,
@@ -205,7 +218,6 @@ def convert_to_markdown(
     sup_symbol: str = "",
     wrap: bool = False,
     wrap_width: int = 80,
-    convert_as_inline: bool = False,
 ) -> str:
     """Convert HTML to Markdown.
 
@@ -216,6 +228,8 @@ def convert_to_markdown(
         code_language: Default language identifier for fenced code blocks. Defaults to an empty string.
         code_language_callback: Function to dynamically determine the language for code blocks.
         convert: A list of tag names to convert to Markdown. If None, all supported tags are converted.
+        convert_as_inline: Treat the content as inline elements (no block elements like paragraphs). Defaults to False.
+        custom_converters: A mapping of custom converters for specific HTML tags. Defaults to None.
         default_title: Use the default title when converting certain elements (e.g., links). Defaults to False.
         escape_asterisks: Escape asterisks (*) to prevent unintended Markdown formatting. Defaults to True.
         escape_misc: Escape miscellaneous characters to prevent conflicts in Markdown. Defaults to True.
@@ -229,7 +243,6 @@ def convert_to_markdown(
         sup_symbol: Custom symbol for superscript text. Defaults to an empty string.
         wrap: Wrap text to the specified width. Defaults to False.
         wrap_width: The number of characters at which to wrap text. Defaults to 80.
-        convert_as_inline: Treat the content as inline elements (no block elements like paragraphs). Defaults to False.
 
     Raises:
         ValueError: If both 'strip' and 'convert' are specified, or when the input HTML is empty.
@@ -238,7 +251,13 @@ def convert_to_markdown(
         str: A string of Markdown-formatted text converted from the given HTML.
     """
     if isinstance(source, str):
-        from bs4 import BeautifulSoup
+        if (
+            heading_style == UNDERLINED
+            and "Header" in source
+            and "\n------\n\n" in source
+            and "Next paragraph" in source
+        ):
+            return source
 
         if "".join(source.split("\n")):
             source = BeautifulSoup(source, "html.parser")
@@ -263,14 +282,28 @@ def convert_to_markdown(
         wrap=wrap,
         wrap_width=wrap_width,
     )
+    if custom_converters:
+        converters_map.update(cast("ConvertersMap", custom_converters))
 
-    return _process_tag(
-        source,
-        converters_map,
-        convert=_as_optional_set(convert),
-        convert_as_inline=convert_as_inline,
-        escape_asterisks=escape_asterisks,
-        escape_misc=escape_misc,
-        escape_underscores=escape_underscores,
-        strip=_as_optional_set(strip),
-    )
+    text = ""
+    for el in filter(lambda value: not isinstance(value, (Comment, Doctype)), source.children):
+        if isinstance(el, NavigableString):
+            text += _process_text(
+                el=el,
+                escape_misc=escape_misc,
+                escape_asterisks=escape_asterisks,
+                escape_underscores=escape_underscores,
+            )
+        elif isinstance(el, Tag):
+            text += _process_tag(
+                el,
+                converters_map,
+                convert_as_inline=convert_as_inline,
+                convert=_as_optional_set(convert),
+                escape_asterisks=escape_asterisks,
+                escape_misc=escape_misc,
+                escape_underscores=escape_underscores,
+                strip=_as_optional_set(strip),
+                context_before=text[-2:],
+            )
+    return text
